"""
Dane o projekcie (potrzebne w setup.py i w generowaniu User-Agenta).
"""
__all__ = "__title__", "__version__", "__description__", "__author__", "__license__"
__title__ = "pymondis"
__version__ = "1.0.0.b2.post1"
__description__ = "Nieoficjalny wrapper Quatromondis API"
__author__ = "Asapros"
__license__ = "MIT"
